#include "fire.h"

//This class is based on the fire example from the book Special Effects Game Programming with DirectX.  I have done
//slight modifications to it, such as support for 16 bit textures(for my voodoo).  I have also allowed chabged the 
//class so that I have more control over the fire, and changed it so that the fire does not burn on the left/right 
//edges of the texture.

/*Usage:
	allocate
	call CFIRE::Init();
	call CFIRE::Burn to animate the fire
	set this class's FireTex texture to any primitive
*/

CFIRE::CFIRE()
{
	FireTex = NULL;
	FireFrontBuffer = NULL;
	FireBackBuffer = NULL;
}

CFIRE::~CFIRE()
{
	delete[] FireFrontBuffer;
	delete[] FireBackBuffer;

	FireTex->Release();
}

int CFIRE::CalculateFire(int CoolFactor)
{
	for(int y = 0; y < TexSize ; y++)
	{
		for(int x = 0; x < TexSize; x++)
		{
			unsigned char vLeft, vRight, vTop, vBelow;
			int FinalVal;

			int xPlus1, yPlus1, xMinus1, yMinus1;

			xPlus1 = x + 1;
			if(xPlus1 >= TexSize)
				xPlus1 = 0;

			
			yPlus1 = y + 1;
			if(yPlus1 >= TexSize)
				yPlus1 = TexSize - 1;

			xMinus1 = x - 1;
			if(xMinus1 < 0)
				xMinus1 = TexSize -1;

			yMinus1 = y - 1;
			if(yMinus1 < 0)
				yMinus1 = 0;

			vLeft = FireFrontBuffer[(y * TexSize) + xPlus1];
			vRight = FireFrontBuffer[(y * TexSize) + xMinus1];
			vTop = FireFrontBuffer[(yPlus1 * TexSize) + x];
			vBelow = FireFrontBuffer[(yMinus1 * TexSize) + x];

			FinalVal = (vLeft + vRight + vTop + vBelow) / 4;

			FinalVal -= CoolFactor;

			if(FinalVal < 0)
				FinalVal = 0;

			FireBackBuffer[(yMinus1 * TexSize) + x] = FinalVal;
		}
	}

	return OK;
}

CFIRE::CalculateFuelVals(int FuelWidth)
{
	for(int x = 5; x < TexSize -5; x += FuelWidth)
	{
		int y = TexSize - 1;
		int Fuel = FireFrontBuffer[(y * TexSize) + x] + ((rand() %64) - FireDensity);
		
		

		if(Fuel > 255)
			Fuel = 255;

		if(Fuel < 0)
			Fuel = 0;

		for(int i = 0; i < FuelWidth; i++)
		{
			FireBackBuffer[(y * TexSize) + x + i] = Fuel;
		}
	}

	return OK;

}

int CFIRE::Init(int TS, char *PaletteFileName, int FW, int CF, int FD, bool b16, IDirect3DDevice8 *lpDevice)
{
	HRESULT hRet;

	FuelWidth = FW;
	CoolFactor = CF;
	TexSize = TS;
	b16Bit = b16;
	FireDensity = FD;

	hRet = D3DXCreateTextureFromFileEx(lpDevice, PaletteFileName, 0, 0, 1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0, NULL, FirePal, &FireTex);
	if(hRet != D3D_OK)
		return FAIL;

	FireFrontBuffer = new BYTE[TexSize * TexSize];
	FireBackBuffer = new BYTE[TexSize * TexSize];

	for(int i = 0; i < (TexSize * TexSize); i++)
		FireFrontBuffer[i] = FireBackBuffer[i] = NULL;

	CalculateFuelVals(FuelWidth);
	CalculateFire(CoolFactor);


	if(b16Bit)
		hRet = D3DXCreateTexture(lpDevice, TexSize, TexSize, 1, 0, D3DFMT_R5G6B5, D3DPOOL_MANAGED, &FireTex);
	else
		hRet = D3DXCreateTexture(lpDevice, TexSize, TexSize, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &FireTex);
	if(hRet != D3D_OK)
		return E_FAIL;

	if(b16Bit)
	{
		for(int i = 0; i < 255; i++)
		{
			FirePal[i].peRed = ((FirePal[i].peRed * 31) / 255);
			FirePal[i].peGreen = ((FirePal[i].peGreen * 63) / 255);
			FirePal[i].peBlue = ((FirePal[i].peBlue * 31) / 255);
		}
	}

	D3DLOCKED_RECT FireRect;
	ZeroMemory(&FireRect, sizeof(FireRect));
	hRet = FireTex->LockRect(0, &FireRect, NULL, 0);
	if(hRet != D3D_OK)
		return FAIL;

	if(b16Bit)
	{
		WORD *pSurfBits = static_cast<WORD *>(FireRect.pBits);
		int Index = 0;
	
		for(int y = 0; y < TexSize - 1; y++)
		{
			for(int x = 0; x < TexSize ; x++)
			{
				pSurfBits[((y * TexSize) + x)] = (FirePal[FireBackBuffer[((y * TexSize) + x)]].peRed << 11| FirePal[FireFrontBuffer[(y * TexSize) + x]].peGreen << 6 | FirePal[FireFrontBuffer[(y * TexSize) + x]].peBlue);
			}
		}
	}
	else
	{
		BYTE *pSurfBits = static_cast<BYTE *>(FireRect.pBits);
		int Index = 0;
	
		for(int y = 0; y < TexSize - 1; y++)
		{
			for(int x = 0; x < TexSize ; x++)
			{
				pSurfBits[Index++] = FirePal[FireBackBuffer[((y * TexSize) + x)]].peBlue;
				pSurfBits[Index++] = FirePal[FireBackBuffer[((y * TexSize) + x)]].peGreen;
				pSurfBits[Index++] = FirePal[FireBackBuffer[((y * TexSize) + x)]].peRed;				

				pSurfBits[Index++] = 0x00000000;
			}
			Index += FireRect.Pitch - (TexSize * 4);
		}
	}


	hRet = FireTex->UnlockRect(0);
	if(hRet != D3D_OK)
		return FAIL;



	return OK;
}

int CFIRE::Burn()
{
	static bool flip;
	D3DLOCKED_RECT FireRect;
	HRESULT hRet;
	BYTE *Temp;

	CalculateFuelVals(FuelWidth);
	CalculateFire(CoolFactor);

	Temp = FireFrontBuffer;
	FireFrontBuffer = FireBackBuffer;
	FireBackBuffer = Temp;

	ZeroMemory(&FireRect, sizeof(FireRect));
	hRet = FireTex->LockRect(0, &FireRect, NULL, 0);
	if(hRet != D3D_OK)
		return FAIL;

	if(b16Bit)
	{
		WORD *pSurfBits = static_cast<WORD *>(FireRect.pBits);
		int Index = 0;

	
		for(int y = 0; y < TexSize - 1 ; y++)
		{
			for(int x = 0; x < TexSize ; x++)
			{
				pSurfBits[((y * TexSize) + x)] = (FirePal[FireFrontBuffer[((y * TexSize) + x)]].peRed << 11| FirePal[FireFrontBuffer[(y * TexSize) + x]].peGreen << 6 | FirePal[FireFrontBuffer[(y * TexSize) + x]].peBlue);
					
			}
		}
	}
	else
	{
		BYTE *pSurfBits = static_cast<BYTE *>(FireRect.pBits);
		int Index = 0;

	
		for(int y = 0; y < TexSize - 1 ; y++)
		{
			for(int x = 0; x < TexSize ; x++)
			{

				pSurfBits[Index++] = FirePal[FireBackBuffer[((y * TexSize) + x)]].peBlue;
				pSurfBits[Index++] = FirePal[FireBackBuffer[((y * TexSize) + x)]].peGreen;
				pSurfBits[Index++] = FirePal[FireBackBuffer[((y * TexSize) + x)]].peRed;				

				pSurfBits[Index++] = 0x00000000;
			}
			Index += FireRect.Pitch - (TexSize * 4);
		}
	}

	hRet = FireTex->UnlockRect(0);
	if(hRet != D3D_OK)
		return FAIL;

	return OK;
}
