#include "MD3Load.h"

/*This class loads the Quake 3 MD3 format an stores it in Vertex/Index buffers.  I created this so that I could import
models from gMax.  You call CMD3FILE::Load to load you MD3 file, then CMD3FILE::Render to draw the md3 file.

  Usage:
  *allocate
  *call CMD3File::Load()
  *call CMD3File::Render() in between BeginScene() and EndScene()
  deallocate
 */

	//assign an index for our vertices
	WORD index[] = {//FRONT(CW)
					0, 2, 1,	
					1, 2, 3,
					//RIGHT SIDE(CW)
					2, 6, 3,
					3, 6, 7,
					//BACK(CCW)
					4, 5, 6,
					5, 7, 6,
					//LEFT SIDE(CCW)
					0, 1, 4,
					4, 1, 5,
					//TOP(CCW)
					4, 2, 0,
					6, 2, 4,
					//BOTTOM(CW)
					1, 3, 5,
					5, 3, 7};

CMD3FILE::CMD3FILE()
{
}

CMD3FILE::~CMD3FILE()
{
	free(BoneFrames);
	free(Tags);
	for(DWORD i = 0; i < NumMeshes; i++)
	{
		free(Meshes[i].TexFilePath);
		free(Meshes[i].TriIndex);
		free(Meshes[i].TexCoords);
		free(Meshes[i].Verts);
		free(Meshes[i].Frames);
		if(Meshes[i].NumTexs > 0)
		{
			if(Meshes[i].lpTexture != NULL)
				free(Meshes[i].lpTexture);
		}
	}

	free(Meshes);
}

int CMD3FILE::PlayAnimation(WORD FPS)
{
	for(DWORD i = 0; i < NumMeshes; i++)
	{
		Meshes[i].PlayAnimation(FPS);
	}
	return OK;
}

int CMD3FILE::Load(char *FilePath, char *tPath, IDirect3DDevice8 *lpDevice, BYTE Alpha)
{
	TexPath = tPath;
	MD3Stream = fopen(FilePath, "rb");

	if(MD3Stream == NULL)
	{
		return ERR_FILEOPEN;
	}

	ReadHeader();
	ReadMD3();
	MakeD3DGeometry(lpDevice, Alpha);

	fclose(MD3Stream);
	return OK;
}

int CMD3FILE::ReadMD3()
{
	long MeshOffset;

	BoneFrames = (MD3BONEFRAME*)malloc(sizeof(MD3BONEFRAME) * NumBoneFrames);

	//bump up one float(I dont know why)
	float temp;
	fread(&temp, sizeof(float), 1, MD3Stream);	

	for(DWORD i = 0; i < NumBoneFrames; i++)
	{
		fread(&BoneFrames[i].MinS.x, sizeof(float), 1, MD3Stream);
		fread(&BoneFrames[i].MinS.y, sizeof(float), 1, MD3Stream);
		fread(&BoneFrames[i].MinS.z, sizeof(float), 1, MD3Stream);

		fread(&BoneFrames[i].MaxS.x, sizeof(float), 1, MD3Stream);
		fread(&BoneFrames[i].MaxS.y, sizeof(float), 1, MD3Stream);
		fread(&BoneFrames[i].MaxS.z, sizeof(float), 1, MD3Stream);

		fread(&BoneFrames[i].Position.x, sizeof(float), 1, MD3Stream);
		fread(&BoneFrames[i].Position.y, sizeof(float), 1, MD3Stream);
		fread(&BoneFrames[i].Position.z, sizeof(float), 1, MD3Stream);

		fread(&BoneFrames[i].Scale, sizeof(float), 1, MD3Stream);

		fread(&BoneFrames[i].Creator, (sizeof(char) * 16), 1, MD3Stream);

	}
	
	Tags = (MD3TAG*)malloc((sizeof(MD3TAG) * NumTags));
	fseek(MD3Stream, TagEnd, SEEK_SET);

	for(i = 0; i < NumTags; i++)
	{
		fread(&Tags[i].Name, (sizeof(float) * 64), 1, MD3Stream);

		fread(&Tags[i].Position.x, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Position.y, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Position.z, sizeof(float), 1, MD3Stream);

		fread(&Tags[i].Rotation._11, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Rotation._12, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Rotation._13, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Rotation._21, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Rotation._22, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Rotation._23, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Rotation._31, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Rotation._32, sizeof(float), 1, MD3Stream);
		fread(&Tags[i].Rotation._33, sizeof(float), 1, MD3Stream);


	}
	fseek(MD3Stream, TagEnd, SEEK_SET);

	Meshes = (MD3MESH*)malloc((sizeof(MD3MESH) * NumMeshes));

	
	MeshOffset = ftell(MD3Stream);

	fseek(MD3Stream, MeshOffset, SEEK_SET);
	for(i = 0; i < NumMeshes; i++)
	{
		fread(&Meshes[i].MeshID, (sizeof(char) * 4), 1, MD3Stream);
		fread(&Meshes[i].Name, (sizeof(char) * 68), 1, MD3Stream);
		fread(&Meshes[i].NumMeshFrames, sizeof(int), 1, MD3Stream);
		fread(&Meshes[i].NumTexs, sizeof(int), 1, MD3Stream);
		fread(&Meshes[i].NumVertices, sizeof(int), 1, MD3Stream);
		fread(&Meshes[i].NumTriangles, sizeof(int), 1, MD3Stream);
		fread(&Meshes[i].TriStart, sizeof(int), 1, MD3Stream);
		fread(&Meshes[i].HeaderSize, sizeof(int), 1, MD3Stream);
		fread(&Meshes[i].TexCoordsStart, sizeof(int), 1, MD3Stream);
		fread(&Meshes[i].VertexStart, sizeof(int), 1, MD3Stream);
		fread(&Meshes[i].MeshSize, sizeof(int), 1, MD3Stream);

		Meshes[i].CurrentFrame = 0;
		
		Meshes[i].TexFilePath = (char*)malloc((sizeof(char) * 128) * Meshes[i].NumTexs);
		fread(Meshes[i].TexFilePath, (sizeof(char) * 68) * Meshes[i].NumTexs, 1, MD3Stream);
		
		if(i == 0)	//go to the right position in the stream if its the first time
			fseek(MD3Stream, MeshOffset + Meshes[i].TriStart, SEEK_SET);

		Meshes[i].TriIndex = (DWORD*)malloc((sizeof(DWORD) * Meshes[i].NumTriangles) * 3);
		fread(Meshes[i].TriIndex, sizeof(DWORD) * 3 * Meshes[i].NumTriangles, 1, MD3Stream);

		if(i == 0)	//go to the right position in the stream if its the first time
		fseek(MD3Stream, MeshOffset + Meshes[i].TexCoordsStart, SEEK_SET);

		Meshes[i].TexCoords = (float*)malloc(sizeof(float) * Meshes[i].NumVertices * 2);
		fread(Meshes[i].TexCoords, sizeof(float) * Meshes[i].NumVertices * 2, 1, MD3Stream);

		if(i == 0)	//go to the right position in the stream if its the first time
		fseek(MD3Stream, MeshOffset + Meshes[i].VertexStart, SEEK_SET);
		int NumTotalVerts = Meshes[i].NumMeshFrames * Meshes[i].NumVertices;
		Meshes[i].Verts = (MD3VERTEX*)malloc(sizeof(LVERTEX) * NumTotalVerts);

		unsigned char temp;

		for(int j = 0; j < NumTotalVerts; j++)
		{
			fread(&Meshes[i].Verts[j].x, sizeof(signed short), 1, MD3Stream);
			fread(&Meshes[i].Verts[j].y, sizeof(signed short), 1, MD3Stream);
			fread(&Meshes[i].Verts[j].z, sizeof(signed short), 1, MD3Stream);

			fread(&temp, sizeof(unsigned char) * 2, 1, MD3Stream);
			//fread(&temp, sizeof(unsigned char), 1, MD3Stream);
		}
		
	}

	return 1001;
}

int CMD3FILE::ReadHeader()
{
	fread(&IDNum, sizeof(char) * 4, 1, MD3Stream);
	fread(&MD3Version, sizeof(int), 1, MD3Stream);
	fread(&FileName, (sizeof(char) * 68), 1, MD3Stream);
	fread(&NumBoneFrames, sizeof(int), 1, MD3Stream);
	fread(&NumTags, sizeof(int), 1, MD3Stream);
	fread(&NumMeshes, sizeof(int), 1, MD3Stream);
	fread(&HeaderLength, sizeof(int), 1, MD3Stream);
	fread(&TagStart, sizeof(int), 1, MD3Stream);
	fread(&TagEnd, sizeof(int), 1, MD3Stream);
	fread(&FileSize, sizeof(int), 1, MD3Stream);

	return OK;
}

int CMD3FILE::MakeD3DGeometry(IDirect3DDevice8 *lpDevice, BYTE Alpha)
{
	HRESULT hRet;
	for(DWORD i = 0; i < NumMeshes; i++)
	{
		DWORD NumMeshTotalVerts = (Meshes[i].NumVertices * Meshes[i].NumMeshFrames);

		Meshes[i].Frames = (MD3FRAME*)malloc(sizeof(MD3FRAME) * Meshes[i].NumMeshFrames);

		for(DWORD j = 0; j < Meshes[i].NumMeshFrames; j++)
		{
			Meshes[i].Frames[j].FrameNum = j;

			lpDevice->CreateVertexBuffer((Meshes[i].NumVertices * sizeof(LVERTEX)), D3DUSAGE_WRITEONLY, D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1, D3DPOOL_DEFAULT, &Meshes[i].Frames[j].lpMeshVB);

		
			LVERTEX *v;
			Meshes[i].Frames[j].lpMeshVB->Lock(0, 0, (BYTE**)&v, 0);
			for(DWORD k = 0; k < Meshes[i].NumVertices; k++)
			{
				v[k].x = (float)Meshes[i].Verts[(k + (j * Meshes[i].NumVertices))].x;
				v[k].y = (float)Meshes[i].Verts[(k + (j * Meshes[i].NumVertices))].y;
				v[k].z = (float)Meshes[i].Verts[(k + (j * Meshes[i].NumVertices))].z;
	

				v[k].dwColor = D3DCOLOR_ARGB(Alpha, 255, 255, 255);
	
				v[k].tu = Meshes[i].TexCoords[k * 2];
				v[k].tv = Meshes[i].TexCoords[(k * 2) + 1];
	
			}
			Meshes[i].Frames[j].lpMeshVB->Unlock();
		}
	
		lpDevice->CreateIndexBuffer((Meshes[i].NumTriangles * 3) * sizeof(WORD), D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_DEFAULT, &Meshes[i].lpMeshIB);
		WORD *idx;
		Meshes[i].lpMeshIB->Lock(0, 0, (BYTE**)&idx, 0);
		for(j = 0; j < (Meshes[i].NumTriangles * 3); j++)
		{
			idx[j] = (WORD)Meshes[i].TriIndex[j];
		}
		Meshes[i].lpMeshIB->Unlock();

		if(Meshes[i].NumTexs > 0)
		{
			Meshes[i].lpTexture = (IDirect3DTexture8**)malloc((sizeof(IDirect3DTexture8) * Meshes[i].NumTexs));	
			ZeroMemory(Meshes[i].lpTexture, sizeof(Meshes[i].lpTexture));

			if(TexPath != NULL)
			{
				if(strcmp("USEDEFAULT", TexPath) == 0)
				{
					for(j = 0; j < Meshes[i].NumTexs; j++)
					{
						char TempName[68];
		
						for(WORD k = 0; k < 68; k++)
							TempName[k] = Meshes[i].TexFilePath[k + (j * 68)];
		
						hRet = D3DXCreateTextureFromFile(lpDevice, Meshes[i].TexFilePath, &Meshes[i].lpTexture[j]);
						if(hRet != D3D_OK)
							Meshes[i].NumTexs = 0;
					}
				}
			
			else
			{
			
				char TempName[68];
				char *TexFileName;
				WORD k = 0;

				for(j = 67; Meshes[i].TexFilePath[j] != '\\'; j--)
				{
					
					if(Meshes[i].TexFilePath[j] != 0)
					{
						TempName[k] = Meshes[i].TexFilePath[j];
						k++;
					}
				}
				
				TexFileName = (char*)malloc(k + 1);
				ZeroMemory(TexFileName, k + 1);
				for(j = 0; j < k; j++)
				{
					TexFileName[j] = TempName[(k - j) - 1];
				}
				TexFileName[j] = '\0';

				
				for(j = 0; j < Meshes[i].NumTexs; j++)
				{
					sprintf(&Meshes[i].TexFilePath[(j * 128)], "%s\\%s", TexPath, TexFileName);

					hRet = D3DXCreateTextureFromFile(lpDevice, Meshes[i].TexFilePath, &Meshes[i].lpTexture[j]);
					if(hRet != D3D_OK)
						Meshes[i].NumTexs = 0;
				}
				
				free(TexFileName);
			}
				
			}
		}
	
	
	}

	return OK;
}

int CMD3FILE::Render(IDirect3DDevice8 *lpDevice)
{	

	HRESULT hRet;
	for(DWORD i = 0; i < NumMeshes; i++)
	{
		if(Meshes[i].bPlayAnimation == true)
		{
			Meshes[i].AnimCurrentTime = timeGetTime();
			if(Meshes[i].AnimCurrentTime - Meshes[i].AnimOldTime > Meshes[i].AnimDelta)
			{
				Meshes[i].CurrentFrame++;
				Meshes[i].AnimOldTime = Meshes[i].AnimCurrentTime;

				if(Meshes[i].CurrentFrame >= Meshes[i].NumMeshFrames)
					Meshes[i].CurrentFrame = 0;
				
			}
		}

	
		if(Meshes[i].NumTexs > 0)
			lpDevice->SetTexture(0, Meshes[i].lpTexture[0]);
		else
			lpDevice->SetTexture(0, NULL);

		lpDevice->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1);	//set the default shader
		lpDevice->SetStreamSource(0, Meshes[i].Frames[Meshes[i].CurrentFrame].lpMeshVB, sizeof(LVERTEX));
		lpDevice->SetIndices(Meshes[i].lpMeshIB, 0);
		hRet = lpDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, Meshes[i].NumVertices, 0, Meshes[i].NumTriangles);
		if(hRet != D3D_OK)
			return 0;
	}
	return OK;
}

int MD3MESH::SetFrame(DWORD Frame)
{

	if(Frame >= NumMeshFrames)
		return 0;

	CurrentFrame = Frame;
	return 1;
}

MD3MESH::PlayAnimation(WORD FPS)
{
	if(NumMeshFrames <= 1)
		return ERR_NOANIMATION;

	bPlayAnimation = true;
	AnimDelta = 1000 / FPS;
	return OK;
}

	




