#include "bmpfont.h"

//This is a bitmap loading class that I made so that I could display text in Direct3D.  I thought the D3DXFont 
//engine was a little too complex, and found no documentation on it so I decided to write my own.  This basically
//loads a bitmap containing all possible font characters, along with a text file containg the actual character of each
//bitmap character.  The class prints a string by looking at each character for the string and finding the appropiate texture.
//When and if the correct texture is found, a quad is drawn with that texture in orthgonal mode.  This procedure is 
//repeated for each character in the string.
/*Usage:
	First call CBMPFONT::Init, to load the font files(the bitmap, and characters)
	Then call CBMPFONT::print, when yu want to print a string
*/
CBMPFONT::CBMPFONT()
{	
}

CBMPFONT::~CBMPFONT()
{	
	delete[] Chars;
}

bool CBMPFONT::Init(char *FilePath, WORD FontColor, IDirect3DDevice8 *lpDevice)
{	
	char FontDescPath[32], FontBMPPath[32];
	int NumChars, CWidth, CHeight;
	IDirect3DSurface8 *lpCharSurf;
	D3DLOCKED_RECT LockedRect;
	WORD *pSurfBits;
	WORD bmpBPP;
	int bmpLeftShift, bmpRightShift;
	

	sprintf(FontDescPath, "%s.fon", FilePath);
	sprintf(FontBMPPath, "%s.bmp", FilePath);

	FILE *fleStream;

	DWORD		bmpWidth, bmpHeight,  bmpDataOffset, bmpCompression, bmpDataSize;
	BYTE		bmpPalette[8];

	BYTE	*bmpBits;

	fleStream = fopen(FontBMPPath, "rb");
	if(!fleStream)
		return false;


	fseek(fleStream, 0x000A, SEEK_SET);
	fread(&bmpDataOffset, sizeof(DWORD), 1, fleStream);

	fseek(fleStream, 0x0012, SEEK_SET);
	fread(&bmpWidth, sizeof(DWORD), 1, fleStream);
	fread(&bmpHeight, sizeof(DWORD), 1, fleStream);

	fseek(fleStream, 0x001C, SEEK_SET);
	fread(&bmpBPP, sizeof(WORD), 1, fleStream);
	if(bmpBPP != 8)
		if(bmpBPP != 4)
			return false;

	fread(&bmpCompression, sizeof(DWORD), 1, fleStream);
	if(bmpCompression)
		return false;

	fread(&bmpDataSize, sizeof(DWORD), 1, fleStream);

	fseek(fleStream, 0x0036, SEEK_SET);

	if(FontColor == NULL)
	{
		fread(bmpPalette, sizeof(BYTE) * 8, 1, fleStream);
	}
	

	fseek(fleStream, bmpDataOffset, SEEK_SET);

	//temp vars for flipping
	int j = 0;
	BYTE *tmp;
		
	bmpBits = (BYTE*)malloc(sizeof(BYTE) * bmpDataSize);
	fread(bmpBits, sizeof(BYTE) * bmpDataSize, 1, fleStream);

	//flip bytes
	tmp = (BYTE*)malloc(sizeof(BYTE) * bmpDataSize);
	for(int y = bmpHeight - 1; y > -1; --y)
	{
		for(int x = 0; x < bmpWidth; ++x)
		{
			tmp[(j * bmpWidth) + x] = bmpBits[(y * bmpWidth) + x];
		}
		j++;
	}
	free(bmpBits);
	bmpBits = tmp;

	fclose(fleStream);

	fleStream = fopen(FontDescPath, "rt");

	fscanf(fleStream, "%d", &NumChars);
	fscanf(fleStream, "%d", &CWidth);
	fscanf(fleStream, "%d", &CHeight);

	CharWidth = (float)CWidth;
	CharHeight = (float)CHeight;

	Chars = new CBMPCHAR[NumChars];	//create a structure that holds all character elements

	for(int i = 0; i < NumChars; i++)	//fill character 
	{
		fread(&Chars[i].Character, sizeof(char), 1, fleStream);
		Chars[i].IndexNum = i;
		
		bmpLeftShift = (CWidth * i);
		bmpRightShift = (bmpLeftShift + CWidth);
		
		Chars[i].Tex = NULL;
		ZeroMemory(&lpCharSurf, sizeof(IDirect3DSurface8));
		D3DXCreateTexture(lpDevice, CWidth, CHeight, 1, 0, D3DFMT_A4R4G4B4, D3DPOOL_MANAGED, &Chars[i].Tex);

		Chars[i].Tex->GetSurfaceLevel(0, &lpCharSurf);
		lpCharSurf->LockRect(&LockedRect, NULL, 0);
		pSurfBits = static_cast<WORD*>(LockedRect.pBits);

		for(int y = 0; y < CharHeight; y++)
		{
			for(int x = 0; x < CharWidth; x++)
			{
				if(bmpBits[((y * bmpWidth) + bmpLeftShift) + x] == 0)	//palette index = 0
					pSurfBits[((y * CWidth) + x)] = 0x0000;
				else
				{
					if(FontColor == NULL)
					{
						int PalIndex = bmpBits[((y * bmpWidth) + bmpLeftShift) + x];
						BYTE b = bmpPalette[PalIndex * 4];
						BYTE g = bmpPalette[(PalIndex * 4) + 1];
						BYTE r = bmpPalette[(PalIndex * 4) + 2];
						pSurfBits[(y * CWidth) + x] = (255 | r << 4 | g << 8 | b << 12);
					}
					else
						pSurfBits[(y * CWidth) + x] = FontColor;
				}
			}
		}

		lpCharSurf->UnlockRect();


		CharMap.insert(make_pair(Chars[i].Character, Chars[i]));
	}
	
	free(tmp);
	fclose(fleStream);

	return true;
}

bool CBMPFONT::print(char *Text, float x, float y, float Scale, LPDIRECT3DDEVICE8 lpDevice)
{
	DWORD OldAlphaBlendEnable;
	x = x / Scale;
	y = y / Scale;

	int numChars = strlen(Text);
	if(!numChars)
		return true;

	//get all transform states so we can set them back after were done
	D3DMATRIX projMat, viewMat, worldMat;

	lpDevice->GetTransform(D3DTS_PROJECTION, &projMat);
	lpDevice->GetTransform(D3DTS_VIEW, &viewMat);
	lpDevice->GetTransform(D3DTS_WORLD, &worldMat);

	int TextLen = strlen(Text);
	int NumChars = strlen(Text);

	DWORD MinFilterTexStage;
	SIMPLEVERTEX TexQuad[4];

    // Enable alpha testing (skips pixels with less than a certain alpha.)
	lpDevice->GetRenderState(D3DRS_ALPHABLENDENABLE, &OldAlphaBlendEnable);
	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
   
	lpDevice->SetRenderState( D3DRS_ALPHATESTENABLE, TRUE );
    lpDevice->SetRenderState( D3DRS_ALPHAREF,   0x99);
    lpDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_GREATER);

	lpDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);	//Using D3DTEXF_LINEAR gives the text a less "boxy" look than setting the magfilter to D3DTEXF_NONE, but D3DTEXF_NONE gives the exact image of the original bitmap image

	lpDevice->GetTextureStageState(0, D3DTSS_MINFILTER, &MinFilterTexStage);
	lpDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_NONE);
	lpDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);
   


	TexQuad[0].x = 0.0f;
	TexQuad[0].y = CharHeight;
	TexQuad[0].z = 0.0f;

	TexQuad[0].dwColor = D3DCOLOR_RGBA(255, 255, 255, 255);

	TexQuad[0].tu = 0.0f;
	TexQuad[0].tv = 0.0f;

	TexQuad[1].x = 0.0f;
	TexQuad[1].y = 0.0f;
	TexQuad[1].z = 0.0f;

	TexQuad[1].dwColor = D3DCOLOR_RGBA(255, 255, 255, 255);

	TexQuad[1].tu = 0.0f;
	TexQuad[1].tv = 1.0f;

	TexQuad[2].x = CharWidth;
	TexQuad[2].y = CharHeight ;
	TexQuad[2].z = 0.0f;

	TexQuad[2].dwColor = D3DCOLOR_RGBA(255, 255, 255, 255);

	TexQuad[2].tu = 1.0f;
	TexQuad[2].tv = 0.0f;

	TexQuad[3].x = CharWidth;
	TexQuad[3].y = 0.0f;
	TexQuad[3].z = 0.0f;

	TexQuad[3].dwColor = D3DCOLOR_RGBA(255,  255, 255, 255);

	TexQuad[3].tu = 1.0f;
	TexQuad[3].tv = 1.0f;
	
	D3DXMATRIX ProjMat, WorldMat, ViewMat;

	D3DVIEWPORT8 viewport;
	lpDevice->GetViewport(&viewport);

	D3DXMatrixOrthoLH(&ProjMat, viewport.Width, viewport.Height, 0.0f, 1.0f);
	
	D3DXMatrixScaling(&ViewMat, Scale, Scale, Scale);
	lpDevice->SetTransform(D3DTS_VIEW, &ViewMat);
	lpDevice->SetTransform(D3DTS_PROJECTION, &ProjMat);

	map<char, CBMPCHAR>::iterator it;

	for(int i = 0; i < NumChars; i++)
	{
		if(Text[i] != 32 || Text[i] != NULL)
		{
			D3DXMatrixTranslation(&WorldMat, ((x + (i * CharWidth)) - ((viewport.Width / 2)) / Scale), y + (((viewport.Height / 2) - CharHeight) / Scale), 0);
			lpDevice->SetTransform(D3DTS_WORLD, &WorldMat);
		
			it = CharMap.find(Text[i]);
			if(it == CharMap.end())
				lpDevice->SetTexture(0, NULL);
			else
			{
				CBMPCHAR &theChar = it->second;
				lpDevice->SetTexture(0, theChar.Tex); 
	
				lpDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
				lpDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
				lpDevice->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1);
				lpDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, TexQuad, sizeof(SIMPLEVERTEX));
			}
		}
	}
	
	lpDevice->SetRenderState( D3DRS_ALPHATESTENABLE, FALSE);


	//lpDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	lpDevice->SetTextureStageState(0, D3DTSS_MINFILTER, MinFilterTexStage);
	//lpDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR);

	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, OldAlphaBlendEnable);

	//set all transforms to what their original states
	lpDevice->SetTransform(D3DTS_PROJECTION, &projMat);
	lpDevice->SetTransform(D3DTS_VIEW, &viewMat);
	lpDevice->SetTransform(D3DTS_WORLD, &worldMat);

	lpDevice->SetTexture(0, NULL);
	

	return true;
}
	
