#include "d3dconsole.h"

//Creates a text output/input area on the top of the screen so that info about the program can be viewed, and commands
//can be entered.  This class uses my bmpfont class to display the lines of text.  When the user enters a string of 
//text, the console process it, if the string is a command, the input command of the console returns a certian value 
//for each command, so that the commnd may be executed in the main program loop.  If the string isint a command, the
//console just displays the string.  Commands are defined by a / infront of them.  To display a line of text, you use 
//the SendText function.

/*usage:
		First, use CD3DCONSOLE::Init to load all the font stuff
		Then activate the console when needed by setting IsActive to true, in the main program loop
		Then call CD3DCONSOLE::Input if any text has been entered from the keyboard, this function will also execute 
		commands.
		Then call CD3DCONSOLE::Draw to draw the console area/text
*/

CD3DCONSOLE::CD3DCONSOLE()
{
	Active = false;

	//Clear();
	bTransIn = false;
	bTransOut = false;
	Prev = 'x';
	lpBgTex = NULL;
	Font = new CBMPFONT();

};

CD3DCONSOLE::~CD3DCONSOLE()
{
	fclose(OutputStream);

	free(ConOutput);
	free(InputBuffer);

	delete Font;

	if(lpBgTex)
		lpBgTex->Release();
}

bool CD3DCONSOLE::Init(char *BmpFontName, char *BgImage, WORD FontColor, char *OutputFileName, IDirect3DDevice8 *lpDevice)
{
	if(OutputFileName)
		OutputStream = fopen(OutputFileName, "wt");

	//allocate and clear text
	ConOutput = (char*)malloc(sizeof(char) * (MAXCOLS * MAXROWS));
	ZeroMemory(ConOutput, sizeof(char) * (MAXCOLS * MAXROWS));
 
	InputBuffer = (char*)malloc(sizeof(char) * MAXCOLS);
	ZeroMemory(InputBuffer, sizeof(char) * MAXCOLS);
	InputBuffer[0] = '>';
	InputPos = 1;


	IsActive = false;

	if(!Font->Init(BmpFontName, FontColor, lpDevice))
		return false;

	if(BgImage != NULL)
	{
		if(D3DXCreateTextureFromFile(lpDevice, BgImage, &lpBgTex) != D3D_OK)
			return false;
	}

	

	return true;
}

bool CD3DCONSOLE::Draw(IDirect3DDevice8 *lpDevice)
{
	if(IsActive == false)
		return true;

	D3DXMATRIX projMat, viewMat, worldMat;

	D3DXMatrixIdentity(&worldMat);
	D3DXMatrixIdentity(&viewMat);

	lpDevice->GetTransform(D3DTS_PROJECTION, &projMat);
	lpDevice->GetTransform(D3DTS_WORLD, &worldMat);
	lpDevice->GetTransform(D3DTS_VIEW, &viewMat);

	SIMPLEVERTEX	QuadVerts[4];
	D3DVIEWPORT8	ViewPort;


	lpDevice->GetViewport(&ViewPort);

	float t = (ViewPort.Width / 2);
	float pos =  t * -1;

	long CurTime;
	float FadeRatio;

	if(bTransIn)
	{
		
		CurTime = timeGetTime();
		FadeRatio = (float)(CurTime - TransTime) / 250;
		if(FadeRatio > 1.0f)
		{
			bTransIn = false;
			FadeRatio = 1.0f;
		}

		ConAlpha = 128 * FadeRatio;
	}

	if(bTransOut)
	{
		IsActive = false;
		bTransOut = false;
	}

	QuadVerts[0].x = pos;
	QuadVerts[0].y = ViewPort.Height / 2;
	QuadVerts[0].z = 0.0f;

	QuadVerts[0].dwColor = D3DCOLOR_RGBA(255, 255, 255, ConAlpha);

	QuadVerts[0].tu = 0.0f;
	QuadVerts[0].tv = 0.0f;

	QuadVerts[1].x = pos;
	QuadVerts[1].y = 0.0f;
	QuadVerts[1].z = 0.0f;

	QuadVerts[1].dwColor = D3DCOLOR_RGBA(255, 255, 255, ConAlpha);

	QuadVerts[1].tu = 0.0f;
	QuadVerts[1].tv = 1.0f;

	QuadVerts[2].x = ViewPort.Width / 2;
	QuadVerts[2].y = ViewPort.Height / 2 ;
	QuadVerts[2].z = 0.0f;

	QuadVerts[2].dwColor = D3DCOLOR_RGBA(255, 255, 255, ConAlpha);

	QuadVerts[2].tu = 1.0f;
	QuadVerts[2].tv = 0.0f;

	QuadVerts[3].x = ViewPort.Width / 2;
	QuadVerts[3].y = 0.0f;
	QuadVerts[3].z = 0.0f;

	QuadVerts[3].dwColor = D3DCOLOR_RGBA(255,  255, 255, ConAlpha);

	QuadVerts[3].tu = 1.0f;
	QuadVerts[3].tv = 1.0f;

	
	D3DXMATRIX conprojMat, conworldMat, conviewMat;
	D3DXMatrixOrthoLH(&conprojMat, ViewPort.Width, ViewPort.Height, 0.0f, 1.0f);
	D3DXMatrixIdentity(&conworldMat);
	D3DXMatrixIdentity(&conviewMat);

	lpDevice->SetTransform(D3DTS_WORLD, &conworldMat);
	lpDevice->SetTransform(D3DTS_PROJECTION, &conprojMat);
	lpDevice->SetTransform(D3DTS_VIEW, &conviewMat);


	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);

	lpDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	lpDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	if(lpBgTex)
		lpDevice->SetTexture(0, lpBgTex);

	lpDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	lpDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	lpDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
	lpDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);

	lpDevice->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1);
	lpDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, QuadVerts, sizeof(SIMPLEVERTEX));

	numCols = (ViewPort.Width / Font->CharWidth);
	numRows = ((ViewPort.Height / 2) / Font->CharHeight);

	if(numRows > MAXROWS)
		numRows = MAXROWS;
	if(numCols > MAXCOLS)
		numCols = MAXCOLS;

	int y = (numRows - 1);

	char *buffer = (char*)malloc(sizeof(char) * numCols);
	
	for(int rowpos = 0; rowpos < numRows; rowpos++)
	{
		for(int colpos = 0; colpos < numCols; colpos++)
		{
			buffer[colpos] = ConOutput[(rowpos * MAXCOLS) + colpos];
			
			if(colpos >= (numCols - 3))
				buffer[colpos] = '.';
		}
		Font->print(buffer, 0, -(y * Font->CharHeight), 1.0f, lpDevice);
		y--;
	}
	free(buffer);

	Font->print(InputBuffer, 0, -((numRows  - 1) * Font->CharHeight), 1.0f, lpDevice);
			
	lpDevice->SetTransform(D3DTS_PROJECTION, &projMat);
	lpDevice->SetTransform(D3DTS_WORLD, &worldMat);
	lpDevice->SetTransform(D3DTS_VIEW, &viewMat);

	return true;
}

int CD3DCONSOLE::Input(char *in)
{
	if(!IsActive)
		return 0;

	if(strlen(in) == 1)
	{
		if(*in == 8)
		{
			if(InputPos > 1)
				InputPos--;
			InputBuffer[InputPos] = NULL;
			return 0;
		}

		if(*in == 13)
		{
			int Comm = Execute();
			return Comm;
		}

		if(*in == '~' || *in == '`')
			return true;


		if(InputPos < MAXCOLS - 1)
		{
			InputBuffer[InputPos] = *in;
			InputPos++;
		}
	}

	return 0;
}

int CD3DCONSOLE::Execute()
{


	if(InputBuffer[1] == '/')	//command indicator
	{
		if(int Comm = ProcessCommand())
		{
			ClearInputLine();
			return Comm;
		}
	}

	ScrollUp(InputBuffer);		//any typed text
	ClearInputLine();
	

	return 0;
}

bool CD3DCONSOLE::ScrollUp(char *Feedback)
{
	fprintf(OutputStream, "%s\n", Feedback);
	char *temp;
	temp = (char*)malloc(sizeof(char) * (MAXROWS * MAXCOLS));
	memcpy(temp, ConOutput, (MAXROWS * MAXCOLS));

	ZeroMemory(ConOutput, sizeof(char) * (MAXCOLS * MAXROWS));

	for(int rowpos = 1; rowpos < MAXROWS; rowpos++)
	{
		for(int colpos = 0; colpos < MAXCOLS; colpos++)
		{
			ConOutput[((rowpos) * MAXCOLS) + colpos] = temp[((rowpos - 1) * MAXCOLS) + colpos];
			
		}
	}

	for(int i = 0; i < strlen(Feedback); i++)
	{
		if(i >= MAXCOLS)
			break;
		ConOutput[MAXCOLS + i] = Feedback[i];
	}

	ConOutHasChanged = true;	//we must redraw console output

	free(temp);

	return true;

}

bool CD3DCONSOLE::SendText(char *text)
{
	return ScrollUp(text);
}

int CD3DCONSOLE::ProcessCommand()
{
	char *command;

	command = (char *)malloc((sizeof(char) * strlen(InputBuffer)) - 1);
	int i;
	for(i = 2; i < strlen(InputBuffer); i++)
		command[(i - 2)] = InputBuffer[i];
	command[(i - 2)] = 0;

	if(strcmp(command, "quit") == 0)
	{
		SendText("Quiting.....");
		PostQuitMessage(0);
	}
	else if(strcmp(command, "clear") == 0)
		ClearOutput();
	else if(strcmp(command, "help") == 0)
	{
		SendText("Commands:");
		SendText("/quit            -return to operating system");
		SendText("/clear           -clears console");
		SendText("/help            -prints list of commands");
		SendText("/showfps         -shows FPS in upper right corner");
		SendText("/hidefps         -hides FPS counter if it is visible");
		SendText("/waterdetaillow  -changes the ocean water detail to its lowest setting");
		SendText("/waterdetailmed  -changes the ocean water detail to its medium setting");
		SendText("/waterdetailhigh -changes the ocean water detail to its highest setting");
	}

	//external commands
	else if(strcmp(command, "showfps") == 0)
	{
		free(command);
		return CONCOMM_SHOWFPS;
	}
	else if(strcmp(command, "hidefps") == 0)
	{
		free(command);
		return CONCOMM_HIDEFPS;
	}
	else if(strcmp(command, "waterdetaillow") == 0)
	{
		free(command);
		return CONCOMM_WATERDETAILLOW;
	}
	else if(strcmp(command, "waterdetailmed") == 0)
	{
		free(command);
		return CONCOMM_WATERDETAILMED;
	}
	else if(strcmp(command, "waterdetailhigh") == 0)
	{
		free(command);
		return CONCOMM_WATERDETAILHIGH;
	}
	else
	{
		SendText("Error Processing Command");
		free(command);
		return 0;
	}
	free(command);

	return 0;
}

void CD3DCONSOLE::ClearOutput()
{
	ZeroMemory(ConOutput, sizeof(char) * (MAXCOLS * MAXROWS));
}

void CD3DCONSOLE::ClearInputLine()
{
	ZeroMemory(InputBuffer, sizeof(char) * MAXCOLS);
	InputBuffer[0] = '>';
	InputPos = 1;
}

bool CD3DCONSOLE::SetBgImage(IDirect3DTexture8 *BgTex)
{
	if(BgTex)
	{
		lpBgTex = BgTex;
		return true;
	}
	else
		return false;
}

bool CD3DCONSOLE::Activate(bool bTrans)
{
	if(IsActive == true)
		return false;

	IsActive = true;
	if(bTrans)
	{
		bTransIn = true;
		TransTime = timeGetTime();
	}
	return true;
}

bool CD3DCONSOLE::Deactivate(bool bTrans)
{
	if(IsActive == false)
		return false;

	if(bTrans)
	{
		bTransOut = true;
		TransTime = timeGetTime();
	}
	else
		IsActive = false;

	return true;
}


	

		





	

