#include "daudio.h"

/*A very simple DirectAudio wrapper that allows me to stop/play/load a wave file with ease.  Uses a vector of segments,
so that any number a segments can be loaded at the same time, then uses a lookup string to find the correct segment

  Usage:
  allocate
  call CDAUDIO::Init()
  call CDAUDIO::LoadWaveFile()  to load a segment with a wave file
  call CDAUDIO::Play() to play the specified segment
  call CDAUDIO::Stop() to stop the segment
  deallocate when done
  */

CDAUDIO::CDAUDIO()
{
	lpLoader = NULL;
	lpPerformance = NULL;
	CoInitialize(NULL);
}

 CDAUDIO::~CDAUDIO()
{
	for(std::vector<CDAudioSegment *>::iterator i = Segments.begin(); i != Segments.end(); i++)
	{
		CDAudioSegment *ThisSeg = *i;
		lpPerformance->Stop(ThisSeg->Segment, NULL, 0, 0);
		ThisSeg->Segment->Release();
		delete ThisSeg;
	}
	Segments.clear();


	lpPerformance->Stop(NULL, NULL, 0, 0);
	lpPerformance->CloseDown();
	lpPerformance->Release();
	lpLoader->Release();
	CoUninitialize();
}

bool CDAUDIO::Init(HWND hWnd)
{
	HRESULT hRet;
	hRet = CoCreateInstance(CLSID_DirectMusicLoader, NULL, CLSCTX_INPROC, IID_IDirectMusicLoader8, (void**)&lpLoader);
	if(hRet != DS_OK)
		return false;

	hRet = CoCreateInstance(CLSID_DirectMusicPerformance, NULL, CLSCTX_INPROC, IID_IDirectMusicPerformance8, (void**)&lpPerformance);
	if(hRet != DS_OK)
		return false;

	hRet = lpPerformance->InitAudio(NULL, NULL, hWnd, DMUS_APATH_DYNAMIC_STEREO, 64, DMUS_AUDIOF_ALL, NULL);
	if(hRet != DS_OK)
		return false;

	return true;
}

bool CDAUDIO::LoadWaveFile(char *WavFilePath, char *LookupName)
{
	HRESULT hRet;
	CDAudioSegment	*lpSeg;

	lpSeg = new CDAudioSegment;

	hRet = lpPerformance->CreateStandardAudioPath(DMUS_APATH_DYNAMIC_STEREO, 64, true, &lpSeg->APath);
	WCHAR wcFileName[MAX_PATH];
	ConvertStringToWide(wcFileName, WavFilePath, -1);
	hRet = lpLoader->LoadObjectFromFile(CLSID_DirectMusicSegment, IID_IDirectMusicSegment8, wcFileName, (LPVOID*)&lpSeg->Segment);
	if(hRet != DS_OK)
		return false;

	lpSeg->LookupName = LookupName;
	Segments.push_back(lpSeg);

	
	return true;
}

//from microsofts dxutil.h, to convert dmusic loader filepath from string to wide string.
void CDAUDIO::ConvertStringToWide( WCHAR* wstrDestination, const CHAR* strSource, 
                                     int cchDestChar )
{
    if( wstrDestination==NULL || strSource==NULL )
        return;

    if( cchDestChar == -1 )
        cchDestChar = strlen(strSource)+1;

    MultiByteToWideChar( CP_ACP, 0, strSource, -1, 
                         wstrDestination, cchDestChar-1 );

    wstrDestination[cchDestChar-1] = 0;
}

bool CDAUDIO::Play(char *LookupName, bool bRepeat)
{
	HRESULT hRet;
	
	for(std::vector<CDAudioSegment *>::iterator i = Segments.begin(); i != Segments.end(); i++)
	{
		CDAudioSegment *ThisSeg = *i;
		if(!strcmp(LookupName, ThisSeg->LookupName))
		{
			ThisSeg->Segment->Download(lpPerformance);
			if(bRepeat)
				ThisSeg->Segment->SetRepeats(DMUS_SEG_REPEAT_INFINITE);
			//hRet = lpPerformance->PlaySegmentEx(ThisSeg->Segment, DSBPLAY_LOOPING, NULL, 0);
			hRet = lpPerformance->PlaySegmentEx(ThisSeg->Segment, NULL, NULL, DMUS_SEGF_SECONDARY, 0, NULL, NULL, ThisSeg->APath);
			if(hRet != DS_OK)
				return false;
			else
				return true;
		}
	}
	return false;
}

bool CDAUDIO::ChangeVolume(char *LookupName, long lVolume)
{
	HRESULT hRet;
	
	for(std::vector<CDAudioSegment *>::iterator i = Segments.begin(); i != Segments.end(); i++)
	{
		CDAudioSegment *ThisSeg = *i;
		if(!strcmp(LookupName, ThisSeg->LookupName))
		{
			hRet = ThisSeg->APath->SetVolume(lVolume, 0);
			if(hRet != DS_OK)
				return false;
			else
				return true;
		}
	}
	return false;
}

bool CDAUDIO::StopSegment(char *LookupName)
{
	HRESULT hRet;

	for(std::vector<CDAudioSegment *>::iterator i = Segments.begin(); i != Segments.end(); i++)
	{
		CDAudioSegment *ThisSeg = *i;
		if(!strcmp(LookupName, ThisSeg->LookupName))
		{
			hRet = lpPerformance->Stop(ThisSeg->Segment, NULL, 0, 0);
			if(hRet != DS_OK)
				return false;
			else
				return true;
		}
	}
	return false;
}


