#include "explosion.h"

/*Displays an explosion using a particle system and a shockwave ring

  Usage:
  allocate
  call CEXPLOSION::Init()
  call CEXPLOSION::Draw()
  deallocate when done
 */

CEXPLOSION::CEXPLOSION()
{
	ExplosionTime = 0;
}

CEXPLOSION::~CEXPLOSION()
{
	delete ExplosionParts;
	lpShockVB->Release();
	lpShockTex->Release();
}

bool CEXPLOSION::Init(char *SWTexPath, char *PartTexPath, int SWDivs, float SWSize, float SWThickness, float SWYPos, BYTE SWAlpha, IDirect3DDevice8 *lpDevice)
{
	Scale = 1.0f;
	D3DXCreateTextureFromFile(lpDevice, SWTexPath, &lpShockTex);

	lpShockVB = CreateHRing(SWDivs, 5.0f, 2.0f, 5.0f, 255, lpDevice);
	if(!lpShockVB)
		return false;

	ExplosionParts = new CPARTICLESYSTEM;
	if(!ExplosionParts->Init(lpDevice, PartTexPath, 100, 110, D3DXVECTOR3(0.5f, 6.7f, 0.0f), 5.0f, D3DXVECTOR3( -5.0f , -5.0f, -5.0f), D3DXVECTOR3(5.0f, 5.0f, 5.0f)))
		return false;

	ExplosionParts->Update(1000, true);
	

	return true;

}

bool CEXPLOSION::Draw(IDirect3DDevice8 *lpDevice, DWORD dTime)
{
	if(ExplosionTime <= 2000)
	{
		ExplosionTime += dTime;
		D3DXMATRIX WMat, scalemat;
		DWORD OldCullMode, OldAlphaBlendEnable, OldSrcBlend, OldDestBlend;

		ExplosionParts->Update(dTime, false);		
		ExplosionParts->Render(lpDevice);

	
		lpDevice->SetVertexShader(FVF_SHOCKWAVEVERT);
		lpDevice->SetStreamSource(0, lpShockVB, sizeof(VERTSHOCKWAVE));

		lpDevice->SetTexture(0, lpShockTex);

		lpDevice->GetRenderState(D3DRS_CULLMODE, &OldCullMode);

		lpDevice->GetRenderState( D3DRS_ALPHABLENDENABLE, &OldAlphaBlendEnable );
		lpDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

		lpDevice->GetRenderState( D3DRS_SRCBLEND, &OldSrcBlend);
		lpDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_ONE);

		lpDevice->GetRenderState( D3DRS_DESTBLEND, &OldDestBlend);
		lpDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE);

		Scale += 25.0f * (float)(dTime / 1000.0f);
		lpDevice->GetTransform(D3DTS_WORLD, &WMat);
		D3DXMatrixScaling(&scalemat, Scale, 1.0f, Scale);
		lpDevice->SetTransform(D3DTS_WORLD, &scalemat);

		lpDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		lpDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, NumRingVerts / 3);

		lpDevice->SetTransform(D3DTS_WORLD, &WMat);

		lpDevice->SetRenderState(D3DRS_CULLMODE, OldCullMode);

		lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, OldAlphaBlendEnable);
		lpDevice->SetRenderState(D3DRS_SRCBLEND, OldSrcBlend);
		lpDevice->SetRenderState(D3DRS_DESTBLEND, OldDestBlend);

		return true;
	}
	else
		return false;
}

IDirect3DVertexBuffer8 *CEXPLOSION::CreateHRing(int NumDivs, float Size, float Thickness, float yPos, BYTE Alpha, IDirect3DDevice8 *lpDevice)
{
	HRESULT hRet;
	IDirect3DVertexBuffer8 * lpRingVB;
	NumRingVerts = NumDivs * 6;

	hRet = lpDevice->CreateVertexBuffer(NumRingVerts * sizeof(VERTSHOCKWAVE), D3DUSAGE_WRITEONLY, FVF_SHOCKWAVEVERT, D3DPOOL_MANAGED, &lpRingVB);
	if(hRet != D3D_OK)
		return NULL;

	VERTSHOCKWAVE *verts;

	hRet = lpRingVB->Lock(0, 0, (BYTE**)&verts, NULL);
	if(hRet != D3D_OK)
		return NULL;

	float step = 360.0f / NumDivs;

	int VBIndex = 0;
	for(float i = 0.0f; i < 360.0f; i += step)
	{
		float x1 = Size * (float)sin(D3DXToRadian(i));
		float z1 = Size * (float)cos(D3DXToRadian(i));
		float x2 = (Size - Thickness) * (float)sin(D3DXToRadian(i));
		float z2 = (Size - Thickness) * (float)cos(D3DXToRadian(i));
		float x3 = Size * (float)sin(D3DXToRadian(i + step));
		float z3 = Size * (float)cos(D3DXToRadian(i + step));
		float x4 = (Size - Thickness) * (float)sin(D3DXToRadian(i + step));
		float z4 = (Size - Thickness) * (float)cos(D3DXToRadian(i + step));

		verts[VBIndex].x = x2;
		verts[VBIndex].y = yPos;
		verts[VBIndex].z = z2;

		verts[VBIndex].dwColor = D3DCOLOR_ARGB(Alpha, 255, 255, 255);

		verts[VBIndex].tu = 0.0f;
		verts[VBIndex].tv = 1.0f;
		VBIndex++;

		verts[VBIndex].x = x1;
		verts[VBIndex].y = yPos;
		verts[VBIndex].z = z1;

		verts[VBIndex].dwColor = D3DCOLOR_ARGB(Alpha, 255, 255, 255);

		verts[VBIndex].tu = 0.0f;
		verts[VBIndex].tv = 0.0f;
		VBIndex++;

		verts[VBIndex].x = x4;
		verts[VBIndex].y = yPos;
		verts[VBIndex].z = z4;

		verts[VBIndex].dwColor = D3DCOLOR_ARGB(Alpha, 255, 255, 255);

		verts[VBIndex].tu = 1.0f;
		verts[VBIndex].tv = 1.0f;
		VBIndex++;

		verts[VBIndex].x = x1;
		verts[VBIndex].y = yPos;
		verts[VBIndex].z = z1;

		verts[VBIndex].dwColor = D3DCOLOR_ARGB(Alpha, 255, 255, 255);

		verts[VBIndex].tu = 0.0f;
		verts[VBIndex].tv = 0.0f;
		VBIndex++;

		verts[VBIndex].x = x3;
		verts[VBIndex].y = yPos;
		verts[VBIndex].z = z3;

		verts[VBIndex].dwColor = D3DCOLOR_ARGB(Alpha, 255, 255, 255);

		verts[VBIndex].tu = 1.0f;
		verts[VBIndex].tv = 0.0f;
		VBIndex++;

		verts[VBIndex].x = x4;
		verts[VBIndex].y = yPos;
		verts[VBIndex].z = z4;

		verts[VBIndex].dwColor = D3DCOLOR_ARGB(Alpha, 255, 255, 255);

		verts[VBIndex].tu = 1.0f;
		verts[VBIndex].tv = 1.0f;
		VBIndex++;
	}
	hRet = lpRingVB->Unlock();
	if(hRet != D3D_OK)
		return NULL;

	return lpRingVB;
}




