#include "main.h"

/*This is the main part of the code that handles initializing/uninitializing Direct3D, and all my classes as well as 
containg the main program loop*/

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
	MSG			msg;
	WNDCLASS	wndClass;

	wndClass.style = CS_HREDRAW | CS_VREDRAW;
	wndClass.lpfnWndProc = WndProc;
	wndClass.cbClsExtra = 0;
	wndClass.cbWndExtra = 0;
	wndClass.hInstance = hInstance;
	wndClass.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(APPICON));
	wndClass.hCursor = LoadCursor(hInstance, MAKEINTRESOURCE(NOCURSOR));
	wndClass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndClass.lpszMenuName = NULL;
	wndClass.lpszClassName = APP_NAME;

	if(!RegisterClass(&wndClass))
	{
		MessageBox(hWnd, "Error: RegisterClass", "Error", MB_OK);
		return 0;
	}

	hWnd = CreateWindow(APP_NAME, APP_NAME, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);


	ShowWindow(hWnd, iCmdShow);
	UpdateWindow(hWnd);

	Timer = new CTIMER();

	InitObjects();
	Timer->Tick();

	while(msg.message != WM_QUIT)
	{
		if(PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{	
			Timer->Tick();
			UpdateAudio();
			RenderFrame();
			iFPS = 1000 / Timer->dTime;
		}
	}

	CleanUp();
	return msg.wParam;
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wparam, LPARAM lparam)
{

	static HINSTANCE hInstance;
	static int FrameNum;
	int Comm;

	switch(message)
	{
	case WM_DESTROY:
		PostQuitMessage(0);
		break;


	case WM_KEYDOWN:
		Demo->KeyPress = true;
		switch(wparam)
		{
			case VK_ESCAPE:
				PostQuitMessage(0);
				break;

			case VK_UP:
				vT.z = 0.1f;
				break;
			case VK_DOWN:
				vT.z = -0.1f;
				break;
			case VK_RIGHT:
				vT.x = 0.1f;

				break;
			case VK_LEFT:
				vT.x = -0.1f;

				break;


		}
		break;
	case WM_KEYUP:
		Demo->KeyPress = false;
		break;



	case WM_CHAR:

		Comm = lpCon->Input((char*)&wparam);
		switch(Comm)
		{
		case CONCOMM_SHOWFPS:
			bShowFPS = true;
			break;
		case CONCOMM_HIDEFPS:
			bShowFPS = false;
			break;
		case CONCOMM_WATERDETAILLOW:
			Ocean->Init("ocean.bmp", D3DCOLOR_RGBA(0, 0, 200, 128), 1.0f, 25, 25, 10, 10, 0.5f, 0.2f, 0.5f, lpDevice);
			break;
		case CONCOMM_WATERDETAILMED:
			Ocean->Init("ocean.bmp", D3DCOLOR_RGBA(0, 0, 200, 128), 1.0f, 25, 25, 25, 25, 0.5f, 0.2f, 0.5f, lpDevice);
			break;
		case CONCOMM_WATERDETAILHIGH:
			Ocean->Init("ocean.bmp", D3DCOLOR_RGBA(0, 0, 200, 128), 1.0f, 48, 48, 100, 100, 0.5f, 0.2f, 0.5f, lpDevice);
			break;

		default:
			break;
		}

		switch(wparam)
		{
			case '~':
			case '`':
					if(lpCon->IsActive == false)
						lpCon->Activate(true);
					else
						lpCon->Deactivate(true);
			break;
	
		}
		break;



	case WM_MOUSEMOVE:
		if(LOWORD(lparam) > MouseXVal)
			ViewRotX += 0.075f;
		if(LOWORD(lparam) < MouseXVal)
			ViewRotX -= 0.075f;
		if(HIWORD(lparam) > MouseYVal)
			ViewRotY += 0.075f;
		if(HIWORD(lparam) < MouseYVal)
			ViewRotY -= 0.075f;

		MouseXVal = LOWORD(lparam);
		MouseYVal = HIWORD(lparam);
		break;

			
	}
	return DefWindowProc(hWnd, message, wparam, lparam);
}

D3DXMATRIX mat;
void InitObjects()
{
	bool	bWindowed;
	bool	b16Bit;

	int		XRes, YRes, WaterDetail;
	

	if(!ReadConfigFile("config.ini", &XRes, &YRes, &bWindowed, &WaterDetail, &bShowFPS))
	{
		MsgBox("Error Loading Configuration File", "Error", hWnd, true);
		return;
	}


	HRESULT hRet;
	lpD3D = Direct3DCreate8(D3D_SDK_VERSION);

	D3DDISPLAYMODE dm;

	lpD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &dm);

	D3DPRESENT_PARAMETERS	d3dpp;

	if(!bWindowed)
	{
		ZeroMemory(&d3dpp, sizeof(d3dpp));
		d3dpp.SwapEffect = D3DSWAPEFFECT_FLIP;
		d3dpp.Windowed = FALSE;
		d3dpp.BackBufferWidth = XRes;
		d3dpp.BackBufferHeight = YRes;
		d3dpp.BackBufferFormat = D3DFMT_R5G6B5;
		d3dpp.EnableAutoDepthStencil = true;
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
		d3dpp.BackBufferCount = 1;
		d3dpp.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_ONE;
	}
	else
	{
		ZeroMemory(&d3dpp, sizeof(d3dpp));
		d3dpp.SwapEffect = D3DSWAPEFFECT_COPY;
		d3dpp.Windowed = TRUE;
		d3dpp.BackBufferFormat = dm.Format;
		d3dpp.EnableAutoDepthStencil = true;
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
		d3dpp.BackBufferCount = 1;
	}

	hRet = lpD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &lpDevice);
	if(hRet != D3D_OK)
	{
		MsgBox("Error Creating Direct3D Device", "Error", hWnd, true);
		return;
	}

	hRet = lpD3D->CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_R5G6B5, 0,	D3DRTYPE_TEXTURE, D3DFMT_A8R8G8B8);
	if(hRet == D3D_OK)
		b16Bit = false;
	else 
		b16Bit = true;


	lpDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	lpDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

	lpDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	lpDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	lpDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
	lpDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	lpDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR);


	lpCon = new CD3DCONSOLE();
	
	if(!lpCon->Init("font\\FONT_8", NULL, 0xF00F, "conout.log", lpDevice))
		MsgBox("Console Initialization Error", "Error", hWnd, true);

	lpCon->SendText("***4 Elements Contest Entry By Tom Keane***");
	lpCon->SendText("Type '/help' for a list of commands");

	BigFont = new CBMPFONT();
	BigFont->Init("font\\FONT_8", 0xF0B0, lpDevice);

	lpDAudio = new CDAUDIO();
	if(lpDAudio->Init(hWnd))
		lpCon->SendText("DirectAudio Init: OK");
	else
	{
		MsgBox("Error Initializing DirectAudio", "Error", hWnd, true);
		return;
	}

	//init console fire
	lpConFire = new CFIRE();
	if(lpConFire->Init(128, "misc\\firepal.bmp", 2, 2, 30, b16Bit, lpDevice))
		lpCon->SendText("Console Fire Init: OK");
	else
	{
		MsgBox("Error Initializing Fire", "Error", hWnd, true);
		return;
	}
		

	//init sky dome
	lpSkyDome = new CSKYDOME();
	if(lpSkyDome->Init(0, 0, lpDevice, "misc\\skydome.md3", NULL, NULL, NULL))
		lpCon->SendText("Sky Dome Init: OK");
	else
	{
		MsgBox("Error Initializing Sky Dome", "Error", hWnd, true);
		return;
	}

	PartSys= new CPARTICLESYSTEM();
	if(PartSys->Init(lpDevice, "textures\\smoke.bmp", 35, 40, D3DXVECTOR3(0.5f, 6.5f, 0.5f), 1.0f, D3DXVECTOR3( -0.5f , 1.0f, -0.5f), D3DXVECTOR3(0.5f, 1.0f, 0.5f)))
		lpCon->SendText("Particle System Init: OK");
	else
	{
		MsgBox("Particle System Init: FAIL", "Error", hWnd, true);
		return;
	}

	Ocean = new CWATER();
	WaterDetail = 32 * WaterDetail;
	if(Ocean->Init("textures\\ocean.bmp", D3DCOLOR_RGBA(0, 0, 200, 128), 1.0f, 48, 48, WaterDetail, WaterDetail, 0.5f, 0.2f, 0.5f, lpDevice))
		lpCon->SendText("Ocean Animation System Init: OK");
	else
	{
		MsgBox("Ocean Animation System Init: FAIL", "Error", hWnd, true);
		return;
	}

	Volcano = new CTERRAIN();
	if(Volcano->Init("textures\\volcanoh24.bmp", "textures\\volcanotex.bmp", 5, 5, 15.0f, lpDevice))
		lpCon->SendText("Volcano Terrain Generation: OK");
	else
	{
		MsgBox("Volcano Terrain Generation: FAIL", "Error", hWnd, true);
		return;
	}

	VolExplosion = new CEXPLOSION();
	if(VolExplosion->Init("textures\\shockwave.bmp", "textures\\erupdebris.bmp", 64, 1.0f, 0.15f, 5.0f, 255, lpDevice))
		lpCon->SendText("Explosion System Init: OK");
	else
	{
		MsgBox("Explosion System Init: FAIL", "Error", hWnd, true);
		return;
	}

	Demo = new CDEMO();
	if(Demo->Init(Timer, BigFont, VolExplosion, PartSys, Volcano, lpDAudio, lpDevice))
		lpCon->SendText("Demo Play System Init: OK");
	else
	{
		MsgBox("Demo Play System Init: FAIL", "Error", hWnd, true);
		return;
	}


	Volcano->CreateHVertexGrid(&lpOceanFloorVB, &lpOceanFloorIB, -0.5f, 64, 64, 2,2, D3DCOLOR_RGBA(255, 255, 255, 255), lpDevice);

	lpDAudio->LoadWaveFile("sound\\seawind.wav", "wind");
	lpDAudio->LoadWaveFile("sound\\shorewav.wav", "waves");
	lpDAudio->LoadWaveFile("sound\\rumble.wav", "rumble");
	lpDAudio->LoadWaveFile("sound\\explosion.wav", "explosion");
	lpDAudio->LoadWaveFile("sound\\fire.wav", "fire");

	lpDAudio->Play("waves", true);
	lpDAudio->Play("wind", true);
}

void RenderFrame()
{


	if(lpCon->IsActive == true)
	{
		lpCon->SetBgImage(lpConFire->FireTex);
		lpConFire->Burn();
	}

	lpDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_RGBA(0, 0, 200, 0), 1.0f, 0);

	lpDevice->BeginScene();	//we are going to begin drawing polys
	
	DrawScene();
	

	if(bShowFPS == true)
	{
		char cfps[10];
		sprintf(cfps, "%i FPS", iFPS);
		lpCon->Font->print(cfps, 0, 0, 1.0f, lpDevice);	//use the console's font system because its there and i dont need to allocate another text
	}

	lpDevice->EndScene();	//we are done drawing polys

	lpDevice->Present(NULL, NULL, NULL, NULL);	//flip to the visible surface

	PartSys->Update(Timer->dTime, true);

}

void UpdateAudio()
{
	if(Demo->PosVec.y <= 2.5f)
	{
		lpDAudio->ChangeVolume("wind", -9600);
		lpDAudio->ChangeVolume("waves", 0);
	}
	if(Demo->PosVec.y > 2.0f && Demo->PosVec.y <= 5.0f)
	{
		long volval = static_cast<long>((Demo->PosVec.y -2.0f)  * 500.0f) ;
		lpDAudio->ChangeVolume("waves", -volval);
		lpDAudio->ChangeVolume("wind", -2000 + volval);
	}

	if(Timer->fElapsedTime > Demo->fExplosionStartTime + 6.5f)
	{
		if(Demo->PosVec.x < 10.0f && Demo->PosVec.x > -10.0f)
		{
			if(Demo->PosVec.z < 10.0f && Demo->PosVec.z > -10.0f)
			{
				long FireVolX = -4000 + (4000 -(fabs(Demo->PosVec.x) * 400));
				long FireVolZ = -4000 + (4000 - (fabs(Demo->PosVec.z) * 400));
				long FireVol = (FireVolX + FireVolZ) / 2;
				lpDAudio->ChangeVolume("fire", FireVol);
			}
		}
	}
}

void DrawScene()
{

	D3DXMATRIX projmat, viewmat, worldmat;
	lpDevice->SetTransform(D3DTS_VIEW, &Demo->vMat);

	//set up a standard 3D Perspective matrix
	D3DXMatrixPerspectiveFovLH(&projmat, D3DX_PI/6, 4/3, 1.0f, 1000.0f);
	lpDevice->SetTransform(D3DTS_PROJECTION, &projmat);

	if(!Demo->bPlayExplosion)
	{
		
		D3DXMatrixIdentity(&worldmat);
		lpDevice->SetTransform(D3DTS_WORLD, &worldmat);
	}
	else
		lpDevice->SetTransform(D3DTS_WORLD, &Demo->wMat);

	//draw volcano model
	Volcano->Draw(lpDevice);
	lpDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
	if(!Demo->bPlayExplosion)
		Demo->PlayDemo(lpDevice);
	else
	{
		Demo->PlayExplosion(lpDevice);
		lpDevice->SetTransform(D3DTS_WORLD, &Demo->wMat);
	}

	if(Demo->bMouseLook)
	{
		Demo->MouseLook(&vT, ViewRotX, ViewRotY);
		lpDevice->SetTransform(D3DTS_WORLD, &Demo->vMat);
	}

	
	D3DXMatrixIdentity(&worldmat);
	lpDevice->SetTransform(D3DTS_WORLD, &worldmat);
	Ocean->Draw(lpDevice, Timer->dTime);
	lpDevice->SetVertexShader(FVF_TERRAINVERTEX);
	lpDevice->SetTexture(0, Ocean->lpWaterTex);
	lpDevice->SetStreamSource(0, lpOceanFloorVB, sizeof(TERRAINVERTEX));
	lpDevice->SetIndices(lpOceanFloorIB, 0);
	lpDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 4, 0, 8);
	PartSys->Render(lpDevice);

	D3DXMatrixScaling(&worldmat, 0.5f, 0.5f, 0.5f);
	lpDevice->SetTransform(D3DTS_WORLD, &worldmat);
	lpDevice->SetTransform(D3DTS_VIEW, &Demo->vMat);

	lpSkyDome->Draw(lpDevice);
	
	lpCon->Draw(lpDevice);
}



void CleanUp()
{
	if(BigFont)
		delete BigFont;
	if(lpSkyDome)
		delete lpSkyDome;
	if(lpConFire)
		delete lpConFire;
	if(lpCon)
		delete lpCon;
	if(Timer)
		delete Timer;
	if(PartSys)
		delete PartSys;
	if(Ocean)
		delete Ocean;
	if(Volcano)
		delete Volcano;
	if(Demo)
		delete Demo;
	if(VolExplosion)
		delete VolExplosion;

	if(lpDAudio)
		delete lpDAudio;
	if(lpD3D)
		lpD3D->Release();
	if(lpDevice)
		lpDevice->Release();
}



	



