#include "skydome.h"

/*This class draws  + amimates a sky dome over a scene.  This class first loads a dome MD3 file using the md3load class
then It draws and animating cloud texture onto the dome.  There are four cloud layers generated through a simple 
noise generator.  The textures are all of different size.  Each texture is then set on a quad and rendered over the 
the other textures using the texture alpha values onto the main cloud texture which is rendered onto the dome over the scene.

  Usage:
  allocate
  *call CSKYDOME::Init()
  *call CSKYDOME::Render() in between BeginScene() and EndScene()
  *deallocate when done
 */


CSKYDOME::CSKYDOME()
{
	AnimFlip = false;
	AnimCloudPos = 0.0f;
}

CSKYDOME::~CSKYDOME()
{
	lpFrontTex = NULL;
	lpBackTex = NULL;
	lpTopTex = NULL;
	lpBottomTex = NULL;
	lpLeftTex = NULL;
	lpRightTex = NULL;
	DomeTex = NULL;
	lpOct128 = NULL;
	lpOct64 = NULL;
	lpOct32 = NULL;
	lpOct16 = NULL;
	lpCloudTex = NULL;
	lpCloudZBuff = NULL;
	lpBlendQuadVB = NULL;
	lpSkyBoxVB = NULL;
	lpSkyDomeVB = NULL;
	lpSkyDomeIB = NULL;
	lpCloudZBuffSurf = NULL;
}
bool CSKYDOME::Init(float width, float height, IDirect3DDevice8 *lpDevice, char *SkyDomePath, char *cSBTexPath, int SBTexWidth, int SBTexHeight)
{

	

	if(SkyDomePath != NULL)
	{
		bIsDome = true;
		md3Dome = new CMD3FILE();

		if(!md3Dome->Load(SkyDomePath, NULL, lpDevice, 128))
			return false;

		lpSkyDomeVB = md3Dome->Meshes[0].Frames[0].lpMeshVB;
		lpSkyDomeIB = md3Dome->Meshes[0].lpMeshIB;
		DomeNumVerts = md3Dome->Meshes->NumVertices;
		DomeNumTriangles = md3Dome->Meshes->NumTriangles;

		delete md3Dome;

		DomeTex = CreateCloudTexture(lpDevice);
		
	}
	else
		bIsDome = false;

	return true;
}

bool CSKYDOME::Draw(IDirect3DDevice8 *lpDevice)
{
	if(bIsDome == true)
	{
		lpDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		lpDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

		lpDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
		AnimateClouds(lpDevice);
		lpDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);


		lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);


		lpDevice->SetTexture(0, DomeTex);

		lpDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		lpDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		lpDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
		lpDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);

		lpDevice->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1);	//set the default shader
		lpDevice->SetStreamSource(0, lpSkyDomeVB, sizeof(LVERTEX));
		lpDevice->SetIndices(lpSkyDomeIB, 0);
		lpDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, DomeNumVerts, 0, DomeNumTriangles);
		lpDevice->SetTexture(0, NULL);
	}
	return true;
}

IDirect3DTexture8 *CSKYDOME::CreateCloudTexture(IDirect3DDevice8 *lpDevice)
{
	IDirect3DSurface8 *lpCloudSurf, *lpOldRenderTarget, *lpOldZBuffer;

	lpOct128 = CreateNoiseTexture(128, lpDevice);
	lpOct64 = CreateNoiseTexture(64, lpDevice);
	lpOct32 = CreateNoiseTexture(32, lpDevice);
	lpOct16 = CreateNoiseTexture(16, lpDevice);

	lpDevice->CreateTexture(256, 256, 1, D3DUSAGE_RENDERTARGET, D3DFMT_R5G6B5, D3DPOOL_DEFAULT, &lpCloudTex);
	lpDevice->CreateDepthStencilSurface(800, 600, D3DFMT_D16, D3DMULTISAMPLE_NONE, &lpCloudZBuffSurf);


	lpCloudTex->GetSurfaceLevel(0, &lpCloudSurf);

	lpDevice->GetDepthStencilSurface(&lpOldZBuffer);
	lpDevice->GetRenderTarget(&lpOldRenderTarget);

	lpDevice->SetRenderTarget(lpCloudSurf, NULL);
	lpDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);
	D3DXMATRIX projMat, worldMat, viewMat;
	D3DXMatrixOrthoLH(&projMat, 2, 2, -1, 1);

	D3DXMatrixIdentity(&worldMat);
	lpDevice->SetTransform(D3DTS_WORLD, &worldMat);
	D3DXMatrixIdentity(&viewMat);
	lpDevice->SetTransform(D3DTS_VIEW, &viewMat);

	lpDevice->SetTransform(D3DTS_PROJECTION, &projMat);

	lpDevice->CreateVertexBuffer(4 * sizeof(SVERTEX), D3DUSAGE_DYNAMIC, FVF_SVERTEX, D3DPOOL_DEFAULT, &lpBlendQuadVB);
	SVERTEX *v;
	lpBlendQuadVB->Lock(0, 0, (BYTE**)&v, 0);

	v[0].x = -1.15f;
	v[0].y = -1.15f;
	v[0].z = 0.0f;

	v[0].dwColor = D3DCOLOR_RGBA(255, 255, 255, 255);

	v[0].tu = 0.0f;
	v[0].tv = 1.0f;

	v[1].x = -1.15f;
	v[1].y = 1.15f;
	v[1].z = 0.0f;


	v[1].dwColor = D3DCOLOR_RGBA(255, 255, 255, 255);

	v[1].tu = 0.0f;
	v[1].tv = 0.0f;

	v[2].x = 1.15f;
	v[2].y = 1.15f;
	v[2].z = 0.0f;

	v[2].dwColor = D3DCOLOR_RGBA(255, 255, 255, 255);

	v[2].tu = 1.0f;
	v[2].tv = 0.0f;

	v[3].x = 1.15f;
	v[3].y = -1.15f;
	v[3].z = 0.0f;

	v[3].dwColor = D3DCOLOR_RGBA(255,  255, 255, 255);

	v[3].tu = 1.0f;
	v[3].tv = 1.0f;

	lpBlendQuadVB->Unlock();
	lpDevice->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_RGBA(0, 0, 255, 0), 1.0f, 0);

	lpDevice->BeginScene();	//we are going to begin drawing polys

	lpDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	lpDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	lpDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	lpDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);

	lpDevice->SetStreamSource(0, lpBlendQuadVB, sizeof(SVERTEX));
	lpDevice->SetVertexShader(FVF_SVERTEX);	//set the default shader

	lpDevice->SetTexture(0, lpOct128);
	lpDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);

	lpDevice->SetTexture(0, lpOct64);
	lpDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);

	lpDevice->SetTexture(0, lpOct32);
	lpDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);

	lpDevice->SetTexture(0, lpOct16);
	lpDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
	
	
	lpDevice->EndScene();	//we are done drawing polys

	lpDevice->SetRenderTarget(lpOldRenderTarget, lpOldZBuffer);
	lpDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);

	return lpCloudTex;
}
	


IDirect3DTexture8 *CSKYDOME::CreateNoiseTexture(int tsize, IDirect3DDevice8 *lpDevice)
{
	IDirect3DTexture8 *lpTex;
	D3DLOCKED_RECT lRect;

	ZeroMemory(&lRect, sizeof(lRect));

	if(lpDevice->CreateTexture(tsize, tsize, 1, 0, D3DFMT_A4R4G4B4, D3DPOOL_MANAGED, &lpTex) != D3D_OK)
		return NULL;

	lpTex->LockRect(0, &lRect, NULL, 0);

	WORD *pSurfBits = static_cast<WORD*>(lRect.pBits);

	int ArrayPos = 0;
	for(int y = 0; y < tsize; y++)
	{
		for(int x = 0; x < tsize; x++)
		{
			BYTE Intensity = (BYTE)((1.0f + Rand2Seed(x, y)) * 8);//make a random number using x and y as seed X AND Y VALS HAVE NO RELEVANCE HERE!!!
			//pSurfBits[ArrayPos++] = ((Intensity) << 12 | (Intensity) << 8 | (Intensity) << 4 | (Intensity));

			pSurfBits[ArrayPos++] = (Intensity << 12 | Intensity << 8| Intensity << 4 | Intensity);
		}
		//ArrayPos += lRect.Pitch / 2;	//2 BYTEs in a WORD(pitch is measured in BYTEs)
	}



	lpTex->UnlockRect(0);

	return lpTex;

}

inline float CSKYDOME::Rand2Seed(int a, int b) //create a random number with 2 seeds
{
	int n = a + b * 57;
	n - (n << 13) ^ n;
	return (1.0f - ((n * (n * n * 15731 + 789221) + 1376312589) & 0x7fffffff) / 1073741824.0f);
}

bool CSKYDOME::AnimateClouds(IDirect3DDevice8 *lpDevice)
{
	HRESULT hRet;
	D3DXMATRIX projMat, worldMat, viewMat, OldWorldMat, OldProjMat, OldViewMat;
	IDirect3DSurface8 *lpCloudSurf, *lpOldRenderTarget, *lpOldZBuffer;



	if(AnimCloudPos > 0.15f  | AnimCloudPos < -0.15f)
	{
		if(AnimFlip == true)
			AnimFlip = false;
		else
			AnimFlip = true;
	}
	if(AnimFlip == true)
		AnimCloudPos+= 0.0001f;
	if(AnimFlip == false)
		AnimCloudPos-= 0.0001f;
	

	lpCloudTex->GetSurfaceLevel(0, &lpCloudSurf);

	//get all current device states that we are going to use
	lpDevice->GetTransform(D3DTS_PROJECTION, &OldProjMat);
	lpDevice->GetTransform(D3DTS_WORLD, &OldWorldMat);
	lpDevice->GetTransform(D3DTS_VIEW, &OldViewMat);

	lpDevice->GetDepthStencilSurface(&lpOldZBuffer);
	lpDevice->GetRenderTarget(&lpOldRenderTarget);

	hRet = lpDevice->SetRenderTarget(lpCloudSurf, NULL);
	if(hRet != D3D_OK)
	{
		return false;
	}
	lpDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);

	D3DXMatrixIdentity(&worldMat);
	lpDevice->SetTransform(D3DTS_WORLD, &worldMat);
	D3DXMatrixIdentity(&viewMat);
	lpDevice->SetTransform(D3DTS_VIEW, &viewMat);
	
	D3DXMatrixOrthoLH(&projMat, 2, 2, -1, 1);
	lpDevice->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_RGBA(0, 0, 255, 0), 1.0f, 0);

	lpDevice->SetTransform(D3DTS_PROJECTION, &projMat);

	lpDevice->BeginScene();	//we are going to begin drawing polys

	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true);

	lpDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	lpDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	lpDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	lpDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);

	hRet = lpDevice->SetStreamSource(0, lpBlendQuadVB, sizeof(SVERTEX));
	if(hRet != D3D_OK)
	{
		return false;
	}
	hRet = lpDevice->SetVertexShader(FVF_SVERTEX);	//set the default shader
	if(hRet != D3D_OK)
	{
		return false;
	}

	D3DXMatrixTranslation(&worldMat, AnimCloudPos, AnimCloudPos, 0);

	lpDevice->SetTransform(D3DTS_WORLD, &worldMat);
	lpDevice->SetTexture(0, lpOct128);
	lpDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);


	D3DXMatrixTranslation(&worldMat, -(AnimCloudPos), -(AnimCloudPos), 0);

	lpDevice->SetTransform(D3DTS_WORLD, &worldMat);
	lpDevice->SetTexture(0, lpOct64);
	lpDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);


	D3DXMatrixTranslation(&worldMat, AnimCloudPos, AnimCloudPos, 0);

	lpDevice->SetTransform(D3DTS_WORLD, &worldMat);
	D3DXMatrixTranslation(&worldMat, -(AnimCloudPos), -(AnimCloudPos), 0);
	lpDevice->SetTexture(0, lpOct32);
	lpDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);

	
	D3DXMatrixTranslation(&worldMat, -(AnimCloudPos), -(AnimCloudPos), 0);

	lpDevice->SetTransform(D3DTS_WORLD, &worldMat);
	lpDevice->SetTexture(0, lpOct16);
	lpDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
	
	lpDevice->EndScene();	///we are done drawing polys

	lpDevice->SetRenderTarget(lpOldRenderTarget, lpOldZBuffer);
	lpDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	lpDevice->SetTransform(D3DTS_PROJECTION, &OldProjMat);
	lpDevice->SetTransform(D3DTS_WORLD, &OldWorldMat);
	lpDevice->SetTransform(D3DTS_VIEW, &OldViewMat);

	return true;


}
	


