#include "terrain.h"

//changes a grid of vertices according to the pixel intensity of a bitmap(white = highest, black = lowest)

/*Usage:
	allocate the class
	call CTERRAIN::Init()
	call CTERRAIN::Draw()
	deallocate
*/
	

CTERRAIN::CTERRAIN()
{
	lpTerrTex = NULL;
}

CTERRAIN::~CTERRAIN()
{
	lpTerrIB->Release();
	lpTerrVB->Release();
	lpTerrTex->Release();

	delete[] fHeightIndex;
}

bool CTERRAIN::Init(char *bmpFilePath, char * TerrTexPath, int XSize, int ZSize, float MaxY, IDirect3DDevice8 *lpDevice)
{
	DWORD bmpDataOffset, bmpWidth, bmpHeight, bmpCompression, bmpDataSize;
	WORD bmpBPP;

	MaxHeight = MaxY;

	D3DXCreateTextureFromFile(lpDevice, TerrTexPath, &lpTerrTex);
	FILE *fleStream;

	fleStream = fopen(bmpFilePath, "rb");

	if(!fleStream)
		return false;

	fseek(fleStream, 0x000A, SEEK_SET);
	fread(&bmpDataOffset, sizeof(DWORD), 1, fleStream);

	fseek(fleStream, 0x0012, SEEK_SET);
	fread(&bmpWidth, sizeof(DWORD), 1, fleStream);
	fread(&bmpHeight, sizeof(DWORD), 1, fleStream);

	NumVerts = (bmpWidth * bmpHeight);

	fseek(fleStream, 0x001C, SEEK_SET);
	fread(&bmpBPP, sizeof(WORD), 1, fleStream);

	fread(&bmpCompression, sizeof(DWORD), 1, fleStream);
		if(bmpCompression)
			return false;

	fread(&bmpDataSize, sizeof(DWORD), 1, fleStream);

	fseek(fleStream, bmpDataOffset, SEEK_SET);

	fHeightIndex = new float[NumVerts];

	for(int i = 0; i < NumVerts; i++)
	{
		if(bmpBPP == 8)
		{
			BYTE PalIndex;
			fread(&PalIndex, sizeof(BYTE), 1, fleStream);
			fHeightIndex[i] = (float)((PalIndex / 255.0f) * MaxHeight);
		}
		if(bmpBPP == 24)
		{
			BYTE IntensityR, IntensityG, IntensityB;
			fread(&IntensityB, sizeof(BYTE), 1, fleStream);
			fread(&IntensityG, sizeof(BYTE), 1, fleStream);
			fread(&IntensityR, sizeof(BYTE), 1, fleStream);
			int TotalIntensity = IntensityR + IntensityG + IntensityB;

			fHeightIndex[i] = (float)((TotalIntensity / 767.0f) * MaxHeight);
		}
	}



	if(!CreateHVertexGrid(&lpTerrVB, &lpTerrIB, 0.0f, XSize, ZSize, bmpWidth, bmpHeight, D3DCOLOR_ARGB(255, 255, 255, 255), lpDevice))
		return false;

	return true;
}

bool CTERRAIN::CreateHVertexGrid(IDirect3DVertexBuffer8 **lpGVB, IDirect3DIndexBuffer8 **lpGIB, float YPos, int XSize,int ZSize, int VertsX, int VertsZ, DWORD dwVertColor, IDirect3DDevice8 *lpDevice)
{
	IDirect3DVertexBuffer8 *lpGridVB;
	IDirect3DIndexBuffer8 *lpGridIB;

	HRESULT hRet;

	hRet = lpDevice->CreateVertexBuffer(sizeof(TERRAINVERTEX) * (VertsX * VertsZ), NULL, FVF_TERRAINVERTEX, D3DPOOL_MANAGED, &lpGridVB);
	if(hRet != D3D_OK)
		return false;

	hRet = lpDevice->CreateIndexBuffer(sizeof(WORD) * ((VertsX * VertsZ) * 6), D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED, &lpGridIB);
	if(hRet != D3D_OK)
		return false;

	TERRAINVERTEX *pVert;
	lpGridVB->Lock(0, 0, (BYTE**)&pVert, NULL);
	for(int x = 0; x < VertsX; x++)
	{
		for(int z = 0; z < VertsZ; z++)
		{
			pVert[(x * VertsZ) + z].x = (((((float)x / (float)(VertsX -1)) * XSize) - (XSize / 2)));
			pVert[(x * VertsZ) + z].y = fHeightIndex[(x * VertsZ) + z];
			pVert[(x * VertsZ) + z].z = (((((float)(VertsZ - 1 - z) / (VertsZ - 1)) * ZSize) - ZSize / 2));

			pVert[(x * VertsZ) + z].dwColor = dwVertColor;

			pVert[(x * VertsZ) + z].tu = (float)x / (VertsX - 1);
			pVert[(x * VertsZ) + z].tv = (float)z / (VertsZ - 1);

		}
	}

	lpGridVB->Unlock();

	WORD *pIndex;
	lpGridIB->Lock(0, sizeof(WORD) * ((VertsX * VertsZ) * 6), (unsigned char**)&pIndex, 0);

	int i = 0;
	for(x = 0; x < VertsX - 1; x++)
	{
		for(int z = 0; z < VertsZ -1 ; z++)
		{
			pIndex[i++] = ((z) * VertsZ) + x;
			pIndex[i++] = ((z) * VertsZ) + x + 1;
			pIndex[i++] = ((z + 1) * VertsZ) + x + 1;
			pIndex[i++] = ((z) * VertsZ) + x;
			pIndex[i++] = ((z + 1) * VertsZ) + x + 1;
			pIndex[i++] = ((z + 1) * VertsZ) + x;
		}
	}


	lpGridIB->Unlock();

	*lpGVB = lpGridVB;
	*lpGIB = lpGridIB;

	return true;
}

bool CTERRAIN::Draw(IDirect3DDevice8 *lpDevice)
{
	HRESULT hRet;
	DWORD AlphaEnable, DestBlend, SrcBlend;

	lpDevice->GetRenderState(D3DRS_ALPHABLENDENABLE, &AlphaEnable);
	lpDevice->GetRenderState(D3DRS_DESTBLEND, &DestBlend);
	lpDevice->GetRenderState(D3DRS_SRCBLEND, &SrcBlend);

	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	lpDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	lpDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	lpDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	lpDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	lpDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
	lpDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);

	if(lpTerrTex)
		lpDevice->SetTexture(0, lpTerrTex);

	hRet = lpDevice->SetVertexShader(FVF_TERRAINVERTEX);	//set the default shader
	if(hRet != D3D_OK)
		return false;

	hRet = lpDevice->SetStreamSource(0, lpTerrVB, sizeof(TERRAINVERTEX));
	if(hRet != D3D_OK)
		return false;
	

	hRet = lpDevice->SetIndices(lpTerrIB, 0);	
	if(hRet != D3D_OK)
		return false;

	hRet = lpDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, NumVerts, 0, (NumVerts * 2));
	if(hRet != D3D_OK)
		return false;

	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, AlphaEnable);
	lpDevice->SetRenderState(D3DRS_SRCBLEND, SrcBlend);
	lpDevice->SetRenderState(D3DRS_DESTBLEND, DestBlend);

	return true;
}