#include "water.h"


/*This is a really simple water generataion/animation class.  A grid of vertices is created, and a random z/x location 
is given to each vertex in the grid.  Whenever CWATER::Draw is called, the grid is drawn, and each vertex's x/z axis
is either increased decreased, depending on whether or not it has reach its maximum/minumum values for each axis.

  Usage:
  *allocate
  *call CWATER::Init();
  *call CWATER::Draw()
  *deallocate when done
 */




CWATER::CWATER()
{
	Elements = NULL;
	lpWaterVB = NULL;
}

CWATER::~CWATER()
{
	delete[] Elements;
	lpWaterVB->Release();
	lpWaterIB->Release();
	lpWaterTex->Release();
}

float CWATER::fRandNum(float fMin, float fMax)
{
  if (fMin == fMax) return(fMin);
  float fRandom = (float)rand() / (float)RAND_MAX;
  return((fRandom * (float)fabs(fMax-fMin))+fMin);
}

bool CWATER::CreateHVertexGrid(IDirect3DVertexBuffer8 **lpGVB, IDirect3DIndexBuffer8 **lpGIB, float YPos, int XSize,int ZSize, int VertsX, int VertsZ, DWORD dwVertColor, IDirect3DDevice8 *lpDevice)
{
	IDirect3DVertexBuffer8 *lpGridVB;
	IDirect3DIndexBuffer8 *lpGridIB;

	HRESULT hRet;

	hRet = lpDevice->CreateVertexBuffer(sizeof(WATERVERTEX) * (VertsX * VertsZ), NULL, FVF_WATERVERTEX, D3DPOOL_MANAGED, &lpGridVB);
	if(hRet != D3D_OK)
		return false;

	hRet = lpDevice->CreateIndexBuffer(sizeof(WORD) * ((VertsX * VertsZ) * 6), D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED, &lpGridIB);
	if(hRet != D3D_OK)
		return false;

	WATERVERTEX *pVert;
	lpGridVB->Lock(0, 0, (BYTE**)&pVert, NULL);

	for(int x = 0; x < VertsX; x++)
	{
		for(int z = 0; z < VertsZ; z++)
		{
			Elements[(x * VertsZ) + z].CurX = pVert[(x * VertsZ) + z].x = (((((float)x / (float)(VertsX -1)) * XSize) - (XSize / 2)) + fRandNum(-XRange, XRange));
			Elements[(x * VertsZ) + z].CurY = pVert[(x * VertsZ) + z].y = (YPos + fRandNum(-YRange, YRange));
			Elements[(x * VertsZ) + z].CurZ = pVert[(x * VertsZ) + z].z = (((((float)(VertsZ - 1 - z) / (VertsZ - 1)) * ZSize) - ZSize / 2) + fRandNum(-ZRange, ZRange));

			pVert[(x * VertsZ) + z].dwColor = dwVertColor;

			pVert[(x * VertsZ) + z].tu = (float)x / (XSize - 1);
			pVert[(x * VertsZ) + z].tv = (float)z / (ZSize - 1);

			Elements[(x * VertsZ) + z].MaxX = (((((float)x / (float)(VertsX -1)) * XSize) - (XSize / 2)) + XRange);
			Elements[(x * VertsZ) + z].MinX = (((((float)x / (float)(VertsX -1)) * XSize) - (XSize / 2)) - XRange);

			Elements[(x * VertsZ) + z].MaxY = (YPos + YRange);
			Elements[(x * VertsZ) + z].MinY = (YPos - YRange);

			Elements[(x * VertsZ) + z].MaxZ = (((((float)(VertsZ - 1 - z) / (VertsZ - 1)) * ZSize) - ZSize / 2) + ZRange);
			Elements[(x * VertsZ) + z].MinZ = (((((float)(VertsZ - 1 - z) / (VertsZ - 1)) * ZSize) - ZSize / 2) - ZRange);

		}
	}

	lpGridVB->Unlock();

	WORD *pIndex;
	lpGridIB->Lock(0, sizeof(WORD) * ((VertsX * VertsZ) * 6), (unsigned char**)&pIndex, 0);

	int i = 0;
	for(x = 0; x < VertsX - 1; x++)
	{
		for(int z = 0; z < VertsZ -1 ; z++)
		{
			pIndex[i++] = ((z) * VertsZ) + x;
			pIndex[i++] = ((z) * VertsZ) + x + 1;
			pIndex[i++] = ((z + 1) * VertsZ) + x + 1;
			pIndex[i++] = ((z) * VertsZ) + x;
			pIndex[i++] = ((z + 1) * VertsZ) + x + 1;
			pIndex[i++] = ((z + 1) * VertsZ) + x;
		}
	}


	lpGridIB->Unlock();

	*lpGVB = lpGridVB;
	*lpGIB = lpGridIB;

	return true;
}

bool CWATER::Init(char *cWaterTexPath, DWORD dwWaterColor, float WaterYPos, int WaterWidth, int WaterLength, int VertDensityX, int VertDensityZ, float XRnge, float YRnge, float ZRnge, IDirect3DDevice8* lpDevice)
{
	if(Elements)
		delete[] Elements;

	if(lpWaterVB)
		lpWaterVB->Release();

	Elements = NULL;
	lpWaterVB = NULL;

	if(cWaterTexPath != NULL)
		D3DXCreateTextureFromFile(lpDevice, cWaterTexPath, &lpWaterTex);

	XRange = XRnge;
	YRange = YRnge;
	ZRange = ZRnge;

	NumVertsX = VertDensityX;
	NumVertsZ = VertDensityZ;

	Elements = new WATERELEMENT[(NumVertsX * NumVertsZ)];

	for(int i = 0; i < (NumVertsX * NumVertsZ); i++)
		Elements[i].CurX = Elements[i].CurY = Elements[i].CurZ = 0.0f;


	if(!CreateHVertexGrid(&lpWaterVB, &lpWaterIB, WaterYPos, WaterWidth, WaterLength, VertDensityX, VertDensityZ, dwWaterColor, lpDevice))
		return false;

	

	return true;
}

bool CWATER::Draw(IDirect3DDevice8 *lpDevice, DWORD dwTime)
{

	Update(dwTime);
	HRESULT hRet;
	DWORD AlphaEnable, DestBlend, SrcBlend;

	lpDevice->GetRenderState(D3DRS_ALPHABLENDENABLE, &AlphaEnable);
	lpDevice->GetRenderState(D3DRS_DESTBLEND, &DestBlend);
	lpDevice->GetRenderState(D3DRS_SRCBLEND, &SrcBlend);

	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	lpDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	lpDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	lpDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	lpDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	lpDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
	lpDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);

	lpDevice->SetTexture(0, lpWaterTex);

	hRet = lpDevice->SetVertexShader(FVF_WATERVERTEX);	//set the default shader
	if(hRet != D3D_OK)
		return false;

	hRet = lpDevice->SetStreamSource(0, lpWaterVB, sizeof(WATERVERTEX));
	if(hRet != D3D_OK)
		return false;
	

	hRet = lpDevice->SetIndices(lpWaterIB, 0);	
	if(hRet != D3D_OK)
		return false;

	hRet = lpDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, (NumVertsX * NumVertsZ), 0, ((NumVertsX * NumVertsZ) * 2));
	if(hRet != D3D_OK)
		return false;

	lpDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, AlphaEnable);
	lpDevice->SetRenderState(D3DRS_SRCBLEND, SrcBlend);
	lpDevice->SetRenderState(D3DRS_DESTBLEND, DestBlend);

	return true;
}

bool CWATER::Update(DWORD dwTime)
{
	float fTime = (float)dwTime / 1000;


	for(int i = 0; i < (NumVertsX * NumVertsZ); i++)
	{
		//check and make sure that each vert has not moved beyond its boundries
		if(Elements[i].CurX >= Elements[i].MaxX)
			Elements[i].bXInc = false;
		if(Elements[i].CurX <= Elements[i].MinX)
			Elements[i].bXInc = true;

		if(Elements[i].CurY >= Elements[i].MaxY)
			Elements[i].bYInc = false;
		if(Elements[i].CurY <= Elements[i].MinY)
			Elements[i].bYInc = true;

		if(Elements[i].CurZ >= Elements[i].MaxZ)
			Elements[i].bZInc = false;
		if(Elements[i].CurZ <= Elements[i].MinZ)
			Elements[i].bZInc = true;


		//move each vert pos
		if(Elements[i].bXInc == true)
			Elements[i].CurX += ((0.2f * fTime) * fRandNum(1.0f, -1.0f));
		else
			Elements[i].CurX -= ((0.2f * fTime) * fRandNum(1.0f, -1.0f));

		if(Elements[i].bYInc == true)
			Elements[i].CurY += ((0.2f * fTime) * fRandNum(1.0f, -1.0f));
		else
			Elements[i].CurY -= ((0.2f * fTime) * fRandNum(1.0f, -1.0f));

		if(Elements[i].bZInc == true)
			Elements[i].CurZ += ((0.2f * fTime) * fRandNum(1.0f, -1.0f));
		else
			Elements[i].CurZ -= ((0.2f * fTime) * fRandNum(1.0f, -1.0f));
			
	}

	//set new vert posistions to VB
	WATERVERTEX *pVert;
	lpWaterVB->Lock(0, 0, (BYTE**)&pVert, NULL);
	for(int x = 0; x < NumVertsX; x++)
	{
		for(int z = 0; z < NumVertsZ; z++)
		{
			pVert[(x * NumVertsZ) + z].x = Elements[(x * NumVertsZ) + z].CurX;
			pVert[(x * NumVertsZ) + z].y = Elements[(x * NumVertsZ) + z].CurY;
			pVert[(x * NumVertsZ) + z].z = Elements[(x * NumVertsZ) + z].CurZ;
		}
	}

	lpWaterVB->Unlock();



	return true;
}
	


